package ru.ulstu.is.sbapp.HardwareShop.controller;

import org.springframework.web.bind.annotation.*;
import ru.ulstu.is.sbapp.HardwareShop.models.Category;
import ru.ulstu.is.sbapp.HardwareShop.services.CategoryService;

import javax.validation.Valid;
import javax.xml.crypto.dsig.CanonicalizationMethod;

import java.io.IOException;
import java.util.List;

@RestController
@RequestMapping("/category")
public class CategoryController {
    private final CategoryService categoryService;

    public CategoryController(CategoryService categoryService) {
        this.categoryService = categoryService;
    }

    @GetMapping("/{id}")
    public CategoryDTO getCategory(@PathVariable Long id) {
        return new CategoryDTO(categoryService.findCategory(id));
    }

    @GetMapping
    public List<CategoryDTO> getCategories() {
        return categoryService.findAllCategories().stream()
                .map(CategoryDTO::new)
                .toList();
    }

    @PostMapping
    public CategoryDTO createCategory(@RequestBody @Valid CategoryDTO categoryDTO) throws IOException {
        return new CategoryDTO(categoryService.addCategory(categoryDTO));
    }

    @PutMapping("/{id}")
    public CategoryDTO updateCategory(@PathVariable Long id, @RequestBody @Valid CategoryDTO categoryDTO) {
        return new CategoryDTO(categoryService.updateCategory(id, categoryDTO));
    }

    @DeleteMapping("/{id}")
    public CategoryDTO deleteCategory(@PathVariable Long id) {
        return new CategoryDTO(categoryService.deleteCategory(id));
    }
}
