package ru.ulstu.is.sbapp.HardwareShop.controller;


import org.springframework.boot.web.servlet.ServletComponentScan;
import org.springframework.web.bind.annotation.*;
import ru.ulstu.is.sbapp.HardwareShop.services.ManufacturerService;

import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

@RestController
@RequestMapping("/manufacturer")
public class ManufacturerController {
    private final ManufacturerService manufacturerService;

    public ManufacturerController(ManufacturerService manufacturerService) {
        this.manufacturerService = manufacturerService;
    }

    @GetMapping("/{id}")
    public ManufacturerDTO getManufacturer(@PathVariable Long id) {
        return new ManufacturerDTO(manufacturerService.findManufacturer(id));
    }

    @GetMapping()
    public List<ManufacturerDTO> getManufacturers() {
        return manufacturerService.findAllManufacturers().stream()
                .map(ManufacturerDTO::new)
                .toList();
    }

    @PostMapping
    public ManufacturerDTO createManufacturer(@RequestBody @Valid ManufacturerDTO manufacturerDTO) throws IOException {
        return new ManufacturerDTO(manufacturerService.addManufacturer(manufacturerDTO));
    }

    @PutMapping("/{id}")
    public ManufacturerDTO updateManufacturer(@PathVariable Long id, @RequestBody @Valid ManufacturerDTO manufacturerDTO) throws IOException {
        return new ManufacturerDTO(manufacturerService.updateManufacturer(id, manufacturerDTO));
    }

    @DeleteMapping("/{id}")
    public ManufacturerDTO deleteManufacturer(@PathVariable Long id) {
        return new ManufacturerDTO(manufacturerService.deleteManufacturer(id));
    }
}
