package ru.ulstu.is.sbapp.HardwareShop.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import ru.ulstu.is.sbapp.HardwareShop.models.Product;
import ru.ulstu.is.sbapp.HardwareShop.services.ManufacturerService;
import ru.ulstu.is.sbapp.HardwareShop.services.ProductService;

import javax.validation.Valid;
import java.io.IOException;
import java.util.Base64;
import java.util.List;

@RestController
@RequestMapping("/product")
public class ProductController {

    private final ProductService productService;

    public ProductController(ProductService productService) {
        this.productService = productService;
    }

    @GetMapping("/{id}")
    public ProductDTO getProduct(@PathVariable Long id) {
        return new ProductDTO(productService.findProduct(id));
    }

    @GetMapping
    public List<ProductDTO> getProducts() {
        return productService.findAllProducts().stream()
                .map(ProductDTO::new)
                .toList();
    }

    @PostMapping
    public ProductDTO createProduct(@RequestBody @Valid ProductDTO productDTO) throws IOException{
        return new ProductDTO(productService.addProduct(productDTO));
    }

    @PutMapping("/{id}")
    public ProductDTO updateProduct(@PathVariable Long id, @RequestBody @Valid ProductDTO productDTO) {
        return new ProductDTO(productService.updateProduct(id, productDTO));
    }

    @PostMapping("/{id}/{manufacturer}")
    public void addManufacturer(@PathVariable Long id, @PathVariable Long manufacturer_id) {
        productService.addManufacturersToProduct(id, manufacturer_id);
    }

    @DeleteMapping("/{id}")
    public ProductDTO deleteProduct(@PathVariable Long id) {
        return new ProductDTO(productService.deleteProduct(id));
    }

    @DeleteMapping
    public void deleteAll() {
        productService.deleteAllProducts();
    }
}
