package ru.ulstu.is.sbapp.HardwareShop.models;

import ru.ulstu.is.sbapp.HardwareShop.controller.ProductDTO;

import java.util.List;
import java.util.Objects;
import javax.persistence.*;
import javax.validation.groups.ConvertGroup;

@Entity
public class Product {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "category_id")
    private Category category;


    @ManyToMany
    @JoinTable(name = "products_manufacturers",
            joinColumns = @JoinColumn(name = "product_fk"),
            inverseJoinColumns = @JoinColumn(name = "manufacturer_fk"))
    private List<Manufacturer> manufacturerList;

    @Column
    private Integer price;

    @Column
    private String name;

    @Lob
    private byte[] photo;

    public Product() {

    }

    public Product(Integer price, byte[] photo, String name)
    {
        this.price = price;
        this.photo = photo;
        this.name = name;
    }

    public Product(ProductDTO productDTO){
        this.price = productDTO.getPrice();
        this.photo = productDTO.getPhoto().getBytes();
        this.name = productDTO.getName();
    }

    public Long getId() { return id; }

    public Integer getPrice() { return price; }

    public byte[] getPhoto() { return photo; }

    public String getName() { return name; }

    public void setName() { this.name = name; }

    public void setPrice(Integer price) { this.price = price; }

    public void setPhoto(byte[] photo) { this.photo = photo; }

    public List<Manufacturer> getManufacturerList() { return manufacturerList; }

    public Category getCategory() { return category; }

    public void setCategory(Category category) { this.category = category; }

    public void addManufacturer(Manufacturer manufacturer) {
        manufacturerList.add(manufacturer);
    }

    public void removeManufacturer(Manufacturer manufacturer) {
        manufacturerList.remove(manufacturer);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Product product = (Product) o;
        return Objects.equals(id, product.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    @Override
    public String toString() {
        return "Product{" +
                "id=" + id +
                ", price='" + price + '\'' +
                '}';
    }

}
