package ru.ulstu.is.sbapp.HardwareShop.services;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import ru.ulstu.is.sbapp.HardwareShop.controller.CategoryDTO;
import ru.ulstu.is.sbapp.HardwareShop.controller.ProductDTO;
import ru.ulstu.is.sbapp.HardwareShop.models.Category;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.is.sbapp.HardwareShop.models.Product;
import ru.ulstu.is.sbapp.HardwareShop.repository.CategoryRepository;
import ru.ulstu.is.sbapp.Util.Validation.ValidatorUtil;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Optional;

@Service
public class CategoryService {
    @Autowired
    private final CategoryRepository categoryRepository;
    @Autowired
    private final ValidatorUtil validatorUtil;

    public CategoryService(CategoryRepository categoryRepository, ValidatorUtil validatorUtil) {
        this.categoryRepository = categoryRepository;
        this.validatorUtil = validatorUtil;
    }

    @Transactional
    public Category addCategory(CategoryDTO categoryDTO) throws IOException {
        final Category category = new Category(categoryDTO);
        validatorUtil.validate(category);
        return categoryRepository.save(category);
    }

    @Transactional
    public Category addCategory(String name) {
        final Category category = new Category(name);
        validatorUtil.validate(category);
        return categoryRepository.save(category);
    }

    @Transactional
    public void saveCategory(Category category) {
        categoryRepository.save(category);
    }

    @Transactional(readOnly = true)
    public Category findCategory(Long id) {
        final Optional<Category> client = categoryRepository.findById(id);
        return client.orElseThrow(() -> new CategoryNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public List<Category> findAllCategories() {
        return categoryRepository.findAll();
    }

    @Transactional
    public Category updateCategory(Long id, String name) {
        final Category currentCategory = findCategory(id);
        currentCategory.setName(name);
        validatorUtil.validate(currentCategory);
        return categoryRepository.save(currentCategory);
    }

    @Transactional
    public Category updateCategory(Long id, CategoryDTO categoryDTO) {
        final Category currentCategory = findCategory(id);
        currentCategory.setName(categoryDTO.getName());
        validatorUtil.validate(currentCategory);
        return categoryRepository.save(currentCategory);
    }

    @Transactional
    public Category deleteCategory(Long id) {
        final Category currentCategory = findCategory(id);
        categoryRepository.delete(currentCategory);
        return currentCategory;
    }

    @Transactional
    public void deleteAllCategories() {
        categoryRepository.deleteAll();
    }
}
