package ru.ulstu.is.sbapp.HardwareShop.services;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import ru.ulstu.is.sbapp.HardwareShop.controller.ManufacturerDTO;
import ru.ulstu.is.sbapp.HardwareShop.controller.ProductDTO;
import ru.ulstu.is.sbapp.HardwareShop.models.Category;
import ru.ulstu.is.sbapp.HardwareShop.models.Manufacturer;


import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.is.sbapp.HardwareShop.models.Product;
import ru.ulstu.is.sbapp.HardwareShop.repository.ManufacturerRepository;
import ru.ulstu.is.sbapp.Util.Validation.ValidatorUtil;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Optional;

@Service
public class ManufacturerService {
    @Autowired
    private final ManufacturerRepository manufacturerRepository;

    @Autowired
    private final ValidatorUtil validatorUtil;

    public ManufacturerService(ManufacturerRepository manufacturerRepository, ValidatorUtil validatorUtil) {
        this.manufacturerRepository = manufacturerRepository;
        this.validatorUtil = validatorUtil;
    }

    @Transactional
    public Manufacturer addManufacturer(String name, String address) {
        final Manufacturer manufacturer = new Manufacturer(name, address);
        validatorUtil.validate(manufacturer);
        return manufacturerRepository.save(manufacturer);
    }

    @Transactional
    public Manufacturer addManufacturer(ManufacturerDTO manufacturerDTO) throws IOException {
        final Manufacturer manufacturer = new Manufacturer(manufacturerDTO);
        validatorUtil.validate(manufacturer);
        return manufacturerRepository.save(manufacturer);
    }

    @Transactional
    public void saveManufacturer(Manufacturer manufacturer) {
        manufacturerRepository.save(manufacturer);
    }

    @Transactional(readOnly = true)
    public Manufacturer findManufacturer(Long id) {
        final Optional<Manufacturer> client = manufacturerRepository.findById(id);
        return client.orElseThrow(() -> new ManufacturerNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public List<Manufacturer> findAllManufacturers() {
        return manufacturerRepository.findAll();
    }

    @Transactional
    public Manufacturer updateManufacturer(Long id, String name, String address) {
        final Manufacturer currentManufacturer = findManufacturer(id);
        currentManufacturer.setName(name);
        currentManufacturer.setAddress(address);
        validatorUtil.validate(currentManufacturer);
        return manufacturerRepository.save(currentManufacturer);
    }

    @Transactional
    public Manufacturer updateManufacturer(Long id, ManufacturerDTO manufacturerDTO) {
        final Manufacturer currentManufacturer = findManufacturer(id);
        currentManufacturer.setName(manufacturerDTO.getName());
        currentManufacturer.setAddress(manufacturerDTO.getAddress());
        validatorUtil.validate(currentManufacturer);
        return manufacturerRepository.save(currentManufacturer);
    }

    @Transactional
    public Manufacturer deleteManufacturer(Long id) {
        final Manufacturer currentManufacturer = findManufacturer(id);
        manufacturerRepository.delete(currentManufacturer);
        return currentManufacturer;
    }

    @Transactional
    public void deleteAllManufacturers() {
        manufacturerRepository.deleteAll();
    }
}
