package ru.ulstu.is.sbapp;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import ru.ulstu.is.sbapp.HardwareShop.models.Category;
import ru.ulstu.is.sbapp.HardwareShop.services.CategoryService;

import javax.persistence.EntityNotFoundException;
import java.util.List;

@SpringBootTest
public class JpaCategoryTests {
    private static final Logger log = LoggerFactory.getLogger(JpaCategoryTests.class);

    @Autowired
    private CategoryService categoryService;

    @Test
    void testCategoryCreate() {
        categoryService.deleteAllCategories();
        final Category category = categoryService.addCategory("Электроника");
        log.info(category.toString());
        Assertions.assertNotNull(category.getId());
    }

    @Test
    void testCategoryRead() {
        categoryService.deleteAllCategories();
        final Category category = categoryService.addCategory("Электроника");
        log.info(category.toString());
        final Category findCategory = categoryService.findCategory(category.getId());
        log.info(findCategory.toString());
        Assertions.assertEquals(category, findCategory);
    }

    @Test
    void testCategoryReadNotFound() {
        categoryService.deleteAllCategories();
        Assertions.assertThrows(EntityNotFoundException.class, () -> categoryService.findCategory(-1L));
    }

    @Test
    void testCategoryReadAll() {
        categoryService.deleteAllCategories();
        categoryService.addCategory("Электроника");
        categoryService.addCategory("Электроника 2");
        final List<Category> categoryList = categoryService.findAllCategories();
        log.info(categoryList.toString());
        Assertions.assertEquals(categoryList.size(), 2);
    }

    @Test
    void testCategoryReadAllEmpty() {
        categoryService.deleteAllCategories();
        final List<Category> categoryList = categoryService.findAllCategories();
        log.info(categoryList.toString());
        Assertions.assertEquals(categoryList.size(), 0);
    }
}
