package ru.ulstu.is.sbapp;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import ru.ulstu.is.sbapp.HardwareShop.models.Manufacturer;
import ru.ulstu.is.sbapp.HardwareShop.services.ManufacturerService;

import javax.persistence.EntityNotFoundException;
import java.util.List;

@SpringBootTest
public class JpaManufacturerTests {
    private static final Logger log = LoggerFactory.getLogger(JpaManufacturerTests.class);

    @Autowired
    private ManufacturerService manufacturerService;

    @Test
    void testManufacturerCreate() {
        manufacturerService.deleteAllManufacturers();
        final Manufacturer manufacturer = manufacturerService.addManufacturer("Производитель 1", "ул.Камышинская д.107");
        log.info(manufacturer.toString());
        Assertions.assertNotNull(manufacturer.getId());
    }

    @Test
    void testManufacturerRead() {
        manufacturerService.deleteAllManufacturers();
        final Manufacturer manufacturer = manufacturerService.addManufacturer("Производитель 1", "ул.Камышинская д.107");
        log.info(manufacturer.toString());
        final Manufacturer findManufacturer = manufacturerService.findManufacturer(manufacturer.getId());
        log.info(findManufacturer.toString());
        Assertions.assertEquals(manufacturer, findManufacturer);
    }

    @Test
    void testManufacturerReadNotFound() {
        manufacturerService.deleteAllManufacturers();
        Assertions.assertThrows(EntityNotFoundException.class, () -> manufacturerService.findManufacturer(-1L));
    }

    @Test
    void testManufacturerReadAll() {
        manufacturerService.deleteAllManufacturers();
        manufacturerService.addManufacturer("Производитель 1", "ул.Камышинская д.107");
        manufacturerService.addManufacturer("Производитель 2", "ул.Камышинская д.108");
        final List<Manufacturer> manufacturerList = manufacturerService.findAllManufacturers();
        log.info(manufacturerList.toString());
        Assertions.assertEquals(manufacturerList.size(), 2);
    }

    @Test
    void testManufacturerReadAllEmpty() {
        manufacturerService.deleteAllManufacturers();
        final List<Manufacturer> manufacturerList = manufacturerService.findAllManufacturers();
        log.info(manufacturerList.toString());
        Assertions.assertEquals(manufacturerList.size(), 0);
    }
}
